# -*- coding: UTF-8 -*-
# lvjiyong on 2015/4/20.
import re
import jieba


def search(pattern, txt):
    """txt中搜索pattern"""
    _pattern = re.compile(pattern, re.IGNORECASE | re.MULTILINE)
    _match = re.search(_pattern, txt)
    if _match:
        return _match.group(1)


def replace(pattern, repl, txt):
    _pattern = re.compile(pattern, re.IGNORECASE | re.MULTILINE)
    return re.sub(_pattern, repl, txt)


def match_encoding(html):
    """从文本中获取charset编码"""
    pattern_str = u"text/html;\s*charset=([a-z\d-]{2,10})"
    group = search(pattern_str, html)
    if group is None:
        pattern_str = u'charset="([a-z\d-]{2,10})"'
        group = search(pattern_str, html)
    return group


def find_all(pattern, source):
    """正则查找匹配组集合"""
    p = re.compile(pattern, re.M | re.I)
    m = re.findall(p, source)
    return m


def match(pattern, source, index):
    """查找匹配字符串返回group(1)值"""
    p = re.compile(pattern, re.M)
    _match = p.search(source)
    if _match:
        return _match.group(index)

def search_cookie(html):
    if 'cookie' in html:
        cookie = re.search(re.compile('cookie=[\'"](.+?)[\'"]', re.I | re.M), html)
        if cookie:
            return cookie.group(1)


def zh_cn(html):
    if not isinstance(html, unicode):
        html = unicode(html)
    pattern = u"[^\u4e00-\u9fa5]+"
    pattern = re.compile(pattern, re.I | re.M)
    result = re.sub(pattern, '', html)
    seg_list = jieba.cut(result)
    b = [s for s in seg_list]
    a = set(b)
    return "".join(a)

def gb(html):

    if not isinstance(html, unicode):
        html = unicode(html)
    pattern = u"[^\u4e00-\u9fa5]+"
    pattern = re.compile(pattern, re.I | re.M)
    result = re.sub(pattern, '', html)
    return result

def remove_all_tags(html):
    pattern = u"<[^>]*?>"
    pattern = re.compile(pattern, re.I | re.M)
    result = re.sub(pattern, '', html)
    return result

def remove_all_blank(html):
    pattern = u"\s*"
    pattern = re.compile(pattern, re.I | re.M)
    result = re.sub(pattern, '', html)
    return result


# -*- coding: UTF-8 -*-
# lvjiyong on 2015/4/26.
import re
import jieba

from gelid.extractors import regex
from gelid.extractors import html
from gelid.extractors import score

from gelid import settings
import sys
reload(sys)
sys.setdefaultencoding('utf8')


def readability_content(source, only_content=True):
    """
    使用readability获取内容，需要安装 readability-lxml
    :param source:
    :param only_content:
    :return:
    """

    try:
        from readability.readability import Document

        doc = Document(source)
        if source and re.search(re.compile('<html', re.I | re.M), source):
            readable_article = doc.summary(html_partial=True)
            if readable_article and only_content:
                readable_article = regex.replace(u'<h1>.*?</h1>', '', readable_article)

            for item in settings.NOT_EXIST:
                find = readable_article.find(item)
                if find>100:
                    readable_article = readable_article[0:find]

            readable_article = readable_article.replace(u'&#13;','')
            return readable_article
    except:
        return ''


def distance_content(source, clear=False, article=False, duplicate=False):
    """
    根据行距离及距离中心距离实现主内容计算
    :param source:
    :return:
    """
    if not clear:
        source = html.set_html_clean(source)
    if not article:
        article_content = html.inner_text('article', source)
        if article_content:
            source = article_content

    pattern = u'<div[^>]*>\s*</div>'
    body = regex.replace(pattern, '', source)

    # 过滤所有正文可能使用的标签
    tags = u'(?P<tag>(/?(p|strong|img|font|span|br|h\d|a|b|i|u|pre|center))[\w\W]*?)'

    def holder_tags(content, tag):
        """
        变化指定标签
        :param content:
        :param tag:
        :return:
        """
        pattern = u'<{0}>'.format(tag)
        content = regex.replace(pattern, u'##\g<tag>##', content)
        # content = regex.replace('<{0}>([\w\W]*?>(</\\1>)?'.format(tag), r'##\g<tag>\g<2>\g<tag>##', content)
        # 转换\n为特殊符号┠
        return regex.replace(u'\n', u'┠', content)

    def clear_tags(content, tag):
        """
        清除转化标签
        :param content:
        :param tag:
        :return:
        """
        _pattern = re.compile(u'(##a [\w\W]*?##/a##)|(##{0}##)'.format(tag) + u'|(┠)', re.I)
        content = re.sub(_pattern, r"", content)
        tag = u'[^\u4e00-\u9fa5\ufe30-\uffa0]'
        return regex.replace(tag, u"", content)

    def recover(content, tag):
        """
        还原转化标签
        :param content:
        :param tag:
        :return:
        """
        _pattern = re.compile(u'##{0}##'.format(tag), re.I)
        return re.sub(_pattern, u'<\g<tag>>', content)

    body = holder_tags(body, tags)
    # 其它html标签转换行
    body = regex.replace(u'(</?.*?>)', u'\n', body)
    # 分割内容至数据
    lines = re.split(u'\n', body)

    # 清除保留的标签，以备文字计数
    z = [clear_tags(i, tags) for i in lines]

    def duplicate_line(line):
        tokens = jieba.cut(line)
        tokens = [token for token in tokens]
        tokens = set(tokens)
        line = ''.join(tokens)
        return line

    # 内容分词处理，清除重复词后再计算文字
    if duplicate:
        z = map(duplicate_line, z)

    # 计算行内数据长
    x = [len(i) for i in z]
    max_x = max(x)
    max_t = 0
    body = lines[x.index(max_x)]


    index = 0
    if max_x > 100 and index == 0:
        index = max_x
    # 如果字数少于100字，则根据距离中心点重新计算
    new_body = None
    if max_x < settings.MAX_WORD_RESEARCH:
        # lines.remove(lines[index])
        mid = len(x) / 2
        if index > 0:
            mid = index
        t = []
        i = 0
        for count in x:
            abs_distance = abs(mid - i)
            nx = count * mid / ((abs_distance + mid) + 1)
            t.append(nx)
            i += 1
        max_t = max(t)
        new_body = lines[t.index(max_t)]

    if new_body:
        for w in settings.NOT_EXIST:
            if w in new_body:
                new_body = None
                break
    if new_body:
        body = new_body
        max_x = max_t
    body = recover(body, tags)
    # 替换原始┠为\n
    body = re.sub(u'┠', u'\n', body)
    # 所有标签改小写
    # body = re.sub(r'<.*?>', lambda a: a.group().lower(), body)
    return body

def zh_CN():
    pass

def rank_content(source, clear=False, duplicate=True):
    return rank_content_with_count(source,clear,duplicate)[0]

def rank_content_with_count(source, clear=False, duplicate=True):

    readability_rank_content = readability_content(source)
    readability_rank = score.Score(dict(content=readability_rank_content))
    distance_rank_content = distance_content(source, clear=clear, duplicate=duplicate)
    distance_rank = score.Score(dict(content=distance_rank_content))

    if distance_rank.rank + 5 > readability_rank.rank:
        return distance_rank.article['content'], len(distance_rank.zh_text)
    else:
        return readability_rank.article['content'], len(distance_rank.zh_text)







# -*- coding: UTF-8 -*-
# lvjiyong on 2015/4/19.
from gelid.extractors import html, score, tm, scan, regex, content, page, title, xpath
from gelid.items import article
from gelidhttp import http, request, response

html = html
score = score
tm = tm
scan = scan
regex = regex
content = content
http = http
request = request
response = response
page = page
title = title
xpath = xpath
Article = article.Article


def config(request_timeout):
    http.request_timeout = request_timeout


# -*- coding: UTF-8 -*-
# lvjiyong on 2015/4/25.


def store(func):
    """
    存储信息到page
    :param func:
    :return:
    """
    def modify(*args, **kwargs):
        page = args[0]
        key = func.__name__
        value = page.stores.get(key)
        if not value:
            value = func(*args, **kwargs)
            if value:
                page.stores[key] = value
        return value
    return modify



# -*- coding: utf-8 -*-
import re
from collections import Counter
from urlparse import *

from gelid.extractors import regex
from gelid.extractors.decorator import store


class Link(object):
    """获取页面链接"""
    _pattern = "<a [^>]*?href[\s=]*[\"']?(%s)[\"' ][^>]*?>"

    def __init__(self, page):
        self.page = page
        self.stores = {}

    @property
    @store
    def urls(self):
        """
        获取页面所有链接
        :return:

        """
        html = self.page.html
        link_pattern = self._pattern % "[^>'\" ]*"
        findall = regex.find_all(link_pattern, html)
        if findall:
            findall = list(set(findall))

        hostname = Link.url_domain(self.page.url)
        findall = [urljoin(hostname, url) for url in findall]
        return findall

    @staticmethod
    def url_domain(url):
        """
        获取网址域名
        :param url:
        :return:
        """

        parse = urlparse(url)

        return '%s://%s' % (parse.scheme,parse.hostname)

    @property
    @store
    def domain_urls(self):
        """
        获取页面所有当前域名的链接
        :return:

        """
        hostname = Link.url_domain(self.page.url)

        return filter(lambda x: re.search(hostname, x), self.urls)

    @staticmethod
    def _max_urls(urls):
        """
        获取最大规则的链接集
        :param urls:
        :return:
        """

        pattern = Link.max_pattern(urls)
        return filter(lambda x: re.search(pattern, x), urls)


    @staticmethod
    def create_pattern(url):
        """
        创建链接规则
        :param url:
        :return:
        """
        return re.sub('\d+', '\d+', re.sub('[a-zA-Z]+', '[a-zA-Z]+', url))

    @staticmethod
    def create_domain_pattern(url):
        """
        创建当前链接基于域名的规则
        :param url:
        :return:
        """
        temp = '___--___'
        hostname = Link.url_domain(url)
        url = url.replace(hostname, temp)
        url = Link.create_pattern(url)
        url = url.replace(temp, hostname)
        return url

    @staticmethod
    def max_pattern(urls):
        """
        获取当前链接最大规则
        :param urls:
        :return:
        """
        _urls = []
        for url in urls:
            _urls.append(Link.create_pattern(url))
        c = Counter(_urls)
        ps = c.most_common(1)
        return ps[0][0]

    @property
    @store
    def max_urls(self):
        """
        获取当前页面最大规则链接集
        :return:
        """
        return Link._max_urls(self.urls)

    @property
    @store
    def domain_max_urls(self):
        """
        获取当前页面最大链接集规则
        :return:
        """
        return Link._max_urls(self.domain_urls)

    def urls_by_pattern(self, pattern):
        """
        获取页面所有链接
        :return:

        """
        html = self.page.html
        link_pattern = self._pattern % (pattern or "[^>'\" ]*")
        findall = regex.find_all(link_pattern, html)
        return findall

    def urls_by_full_pattern(self, pattern):
        """
        获取页面所有链接
        :return:

        """
        html = self.page.html
        findall = regex.find_all(pattern or self._pattern, html)
        return findall



# -*- coding: UTF-8 -*-
# lvjiyong on 2015/4/26.

import datetime
import time
import re


def get_time():
    # 获得当前时间
    now = datetime.datetime.now()
    # 转换为指定的格式:
    ftime = now.strftime("%Y-%m-%d %H:%M:%S")
    return ftime


def get_timestamp(ctime=None):
    """获取时间戳"""
    if ctime:
        return time2timestamp(ctime)
    else:
        return time.time()


def timestamp_tostring(stamp):
    if stamp is not None and re.compile('^[\d\.]+$').match(str(stamp)):
        return time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(float(stamp)))
    else:
        return stamp


def time_tostring(ctime=None):
    timestamp = get_timestamp(ctime)
    return timestamp_tostring(timestamp)


def time2timestamp(ctime=None):
    try:
        if isinstance(ctime, time.struct_time):
            timestamp = time.mktime(ctime)
        elif isinstance(ctime, str):
            timestamp = time.mktime(time.strptime(ctime, '%Y-%m-%d %H:%M:%S'))
        elif isinstance(ctime, int):
            timestamp = time.time() + ctime
        else:
            timestamp = time.time()
    except:
        timestamp = time.time()
    return timestamp


# -*- coding: UTF-8 -*-
# lvjiyong on 2015/4/26.
import re
from gelid.extractors import regex
import time


def get_author(txt):
    """查找作者，以作者|记者开头的文本即为作者"""
    tag = u"(作者|记者)"
    author = get_string_some(txt, tag, 3)
    if not author:
        author = get_string_mix(txt, tag)
    return author


def scan(txt, tag):
    """查找"""
    return get_string_mix(txt, u'({0})'.format(tag))


def get_editor(txt):
    """查找编辑"""
    tag = u"(编辑|责编)"
    return get_string_mix(txt, tag)


def get_from(txt):
    """查找来源"""
    tag = u"(来源|来自|出自|摘自|出处)"
    return get_string_mix(txt, tag)


def get_share_count(txt):
    """查找推荐|分享"""
    tag = u"(推荐|分享)"
    return get_search_mix(txt, tag, u"\d+")


def get_comment_count(txt):
    """查找推荐|分享"""
    tag = u"(回贴数?|回复数?|跟贴数?)"
    comment = get_search_mix(txt, tag, u"\d+")
    if not comment:
        tag = u"(人回贴|人回复|人跟贴)"
        comment = get_search_last(txt, tag, u"\d+")
    return comment


def get_string_some(txt, tag, max_len):
    """找多匹配字符串"""
    pn = u'\s*[:：/]\s*(([^\s]{2,' + str(max_len) + u'} )*)'
    txt = get_match_group(txt, tag, pn, 2)
    if txt:
        clear = u"[/，,、|｜:：]"
        txt = regex.replace(clear, " ", txt)
        some = re.split(' ', txt)
        txt = ""
        for s in some:
            slen = len(s)
            if slen > max_len:
                break
            elif slen > 0:
                txt = txt + " " + s
        if len(txt) > 0:
            txt = txt[1:]
        if len(txt) > 2:
            return txt


def get_string_first(txt, tag):
    """优先查找匹配字符串"""
    return get_search_first(txt, tag, u".*?")


def get_string_second(txt, tag):
    """普通查找匹配字符串"""
    return get_search_second(txt, tag, u".*?")


def get_string_mix(txt, tag):
    """混合查找匹配字符串，先优先，后普通"""
    mix = get_string_first(txt, tag)
    if not mix:
        mix = get_string_second(txt, tag)
    return mix


def get_search_first(txt, tag, search):
    """优先查找匹配字符串"""
    pn = u'\s*[:：/]\s*(' + search + u')\s'
    return get_match_group(txt, tag, pn, 2)


def get_search_second(txt, tag, search):
    """普通查找匹配字符串"""
    pn = u'\s*[:：\s]+(' + search + u')\s'
    return get_match_group(txt, tag, pn, 2)


def get_search_last(txt, tag, search):
    """普通查找匹配字符串"""
    pn = u'(' + search + ')\s*'
    return get_match_group_last(txt, tag, pn, 1)


def get_search_mix(txt, tag, search):
    """混合查找匹配字符串，先优先，后普通"""
    mix = get_search_first(txt, tag, search)
    if not mix:
        mix = get_search_second(txt, tag, search)
    return mix


def get_match_group(txt, tag, pn, index):
    """查找匹配字符串返回group(1)值"""
    pattern = tag + pn
    return get_match_base(txt, pattern, index)


def get_match_group_last(txt, tag, pn, index):
    """查找匹配字符串返回group(1)值"""
    pattern = pn + tag
    return get_match_base(txt, pattern, index)


def get_match_base(txt, pattern, index):
    """查找匹配字符串返回group(1)值"""
    return regex.match(pattern, txt, index)

def get_time(txt):
    time_format = u"\
(\d{4})[年\-/ ]{1,2}\
(\d{1,2})[月\-/ ]{1,2}\
(\d{1,2})[日\-/ ]{0,2}\
((\d{1,2})[时点:\-/ ]{1,2})?\
((\d{1,2})[分:\-/ ]{1,2})?\
((\d{1,2})[秒:\-/ ]{0,2})?\
"
    pattern = re.compile(time_format)
    match = pattern.search(txt)
    # 提取时间内容
    if match:
        group = match.groups()
        time_list = list()
        # 提取时间数字生成数组
        index = 0
        for data in group:
            if index in (0, 1, 2, 4, 6, 8):
                if data:
                    time_list.append(int(data))
                else:
                    time_list.append(0)
            index += 1

        str_time = ''
        for data in time_list:
            str_time = u"%s:%s" % (str_time, data)

        str_time = str_time[1:]
        # 生成时间
        try:
            tx = time.strptime(str_time, u'%Y:%m:%d:%H:%M:%S')
        except:
            tx = None
        return tx


# -*- coding: UTF-8 -*-
# lvjiyong on 2015/4/19.

from gelid.extractors import title, content as ex_content, scan, tm, regex, score
from gelid.extractors.page import Page, store, html
from gelidhttp import request
from gelid import settings


class Article(object):
    def __init__(self, source, url, keyword=None,error_callback=None, next_min=50, parse=None):
        self.page = Page(source=source, url=url)
        self.stores = {}
        self.keyword = keyword
        self.request_all = False
        self.zh_count = 0
        self.next_min = next_min
        self.error_callback = error_callback
        self.parse = parse

    def _request(self, url, contents, pages, contents_images):
        """
        获取分页内容
        :param url:
        :param contents:
        :param pages:
        :param contents_images:
        :return:
        """
        if not contents:
            contents = list()

        if not pages:
            pages = list()

        if url not in pages and len(pages) < settings.REQUEST_MAX_PAGES:
            source = request.Request(url=url, error_callback=self.error_callback).response.body_as_unicode()
            if source:
                article = Article(source, url, parse=self.parse)
                _content = article.content
                # 清除之前有的图片
                for image in contents_images:
                    _content = regex.replace(u'<img .*?src=[\'" ]*{0}[\'" ]*.*?>'.format(image), '', _content)

                for image in article.images:
                    if image not in contents_images:
                        contents_images.append(image)
                if _content not in contents:
                    contents.append(_content)
                    pages.append(url)
                    next_page = html.next_page(url, source)
                    if next_page:
                        self._request(next_page, contents, pages, contents_images)

        self.page.stores['images'] = contents_images
        return contents

    def _page_contents(self):
        """
        获取所有分页内容
        :return:
        """
        pages = [self.page.url]
        contents = [self.content]
        contents_images = self.images
        next_page = html.next_page(self.page.url, self.page.html_clean)
        # 仅当有下一页且本内容有图片或汉字大于100字时获取下一页内容
        if next_page and (self.images or self.zh_count>self.next_min):
            return self._request(next_page, contents, pages, contents_images)
        else:
            return contents

    @property
    @store
    def page_contents(self):
        """
        所有分页内容
        :return:
        """
        self.request_all = True
        return self._page_contents()

    @property
    @store
    def title(self):
        """
        基于title文本扫描分析标题
        :return:
        """
        article_title = title.title_in_content(self.page.title, self.page.body)
        return article_title


    @property
    @store
    def content(self):
        if self.parse == 'readability':
            _content =  ex_content.readability_content(self.page.html_clean)
            zh_text = regex.zh_cn(_content)
            self.zh_count = len(zh_text)
        elif self.parse == 'distance':
            _content =  ex_content.distance_content(self.page.html_clean)
            zh_text = regex.zh_cn(_content)
            self.zh_count = len(zh_text)
        else:
            rank_content =  ex_content.rank_content_with_count(self.page.html_clean, clear=True)
            _content = rank_content[0]
            self.zh_count = rank_content[1]
        _content = html.rebuild(_content, self.page.url)
        _content = html.format_content(_content)
        return _content

    @property
    @store
    def author(self):
        return scan.get_author(self.page.txt)

    @property
    @store
    def time_posted(self):
        return tm.get_timestamp(scan.get_time(self.page.txt))

    @property
    @store
    def come_from(self):
        return scan.get_from(self.page.txt)
    @property
    @store
    def description(self):
        desc = regex.remove_all_tags(self.content)
        desc = regex.remove_all_blank(desc)
        if len(desc)>50:
            desc = desc[:50]
        return desc

    @property
    @store
    def images(self):
        return html.images(self.content)

    @property
    @store
    def rank(self):

        if self.request_all:
            _content = u'<p><!--pager--></p>'.join(self.page_contents)
        else:
            _content = self.content

        article = dict(content=_content, title=self.title, keyword=self.keyword, posted_date=self.time_posted)
        stat = score.Score(article)
        return stat.rank


# -*- coding: UTF-8 -*-
# lvjiyong on 2015/4/19.
import re
from urlparse import urlparse, urljoin

import six

from gelid import settings
from gelid.extractors import regex


def inner_text(tag, source):
    """返回标签内内容"""
    if tag and source:
        pattern = re.compile(u'<{0}.*?>([\w\W]*?)</{0}>'.format(tag), re.I | re.M)
        search = re.search(pattern, source)
        if search:
            return search.group(1)


def urls(url):
    """
    返回urls字典
    """
    assert url, u'url不能为空'
    parse = urlparse(url)
    question_index = url.find(u'?')
    url_host = parse.hostname
    url_without_question = url if question_index == -1 else url[0:question_index]
    url_root = url_without_question[0:url_without_question.rfind('/') + 1]
    url_filename = url_without_question[url_without_question.rfind('/') + 1:]
    path_index = url_filename.find('.')
    fileid = url_filename if path_index == -1 else url_filename[0:path_index]
    pattern = re.compile(r"^(.+)[\-_]\d{1,2}$", re.I)
    match = re.search(pattern, fileid)
    if match:
        fileid = match.group(1)
    _urls = {"dir": url_root, "filename": url_filename, "fileid": fileid, 'domain': url_host}
    return _urls


def next_page(url, source, token=u'下一页|下页|&gt;|》|next|翻页|next page'):
    u"""自动分析下一页地址：
    假设分页都是类似的，均与当前URL有一定的规则关联，仅为分页码不一样或有无
    正则获取下一页地址，如果为找到分页，且分页地址符合规则，则为分页
    """
    if not url or not re.match(r'http.+', url):
        raise ValueError(u'url格式错误!')
    _urls = urls(url)
    fileid = _urls["fileid"]
    pattern = u'href=.*?' + fileid + u'[\w\W]{1,200}?</a>'
    m = regex.find_all(pattern, source)
    page_url = None
    if m:
        for mc in m:
            if re.search(token, mc):
                pattern = u'href=\s*[\'"]([^\'"]*?' + fileid + u'[/\-\._][^\'" ]*?)[\'"]'
                mx = re.search(pattern, mc)
                if mx:
                    page_url = mx.group(1)
                    break
                else:
                    pattern = u'href=\s*([^\'"]*?' + fileid + u'[/\-\._][^\'"\s>]*)'
                    mx = re.search(pattern, mc)
                    if mx:
                        page_url = mx.group(1)
                        break

    if page_url:
        page_url = urljoin(url, page_url)
    return page_url


def clear_tags_with_content(tags, content):
    """清除标签组及中间的内容"""
    if isinstance(tags, six.string_types):
        tags = [tags]
    content = reduce(lambda x, y: clear_tag_with_content(y, x), tags, content)
    return content


def clear_tag_with_content(tag, content):
    """清除标签及中间的内容"""
    pattern = u'<({0})[^>]*?>[\w\W]*?</\\1>'.format(tag)
    return regex.replace(pattern, '', content)


def clear_tags(tags, content):
    """清除标签组"""
    if isinstance(tags, six.string_types):
        tags = [tags]
    content = reduce(lambda x, y: clear_tag(y, x), tags, content)
    return content


def clear_tag(tag, content):
    """清除标签"""
    pattern = u'</?{0}.*?>'.format(tag)
    return regex.replace(pattern, '', content)


def txt(content):
    """生成txt，并返回txt"""
    pattern = u'<[^>]*>|\r|\n|\t'
    content = regex.replace(pattern, '', content)
    pattern = u"&nbsp;"
    content = regex.replace(pattern, ' ', content)
    pattern = u"&[\w\d]{2,8}?;"
    content = regex.replace(pattern, '', content)
    return content


def clear_comment(content):
    """清除注释"""
    pattern = u'<!\-[\w\W]+?\->'
    return regex.replace(pattern, '', content)


def set_html_clean(content):
    """生成html无用标签清洗后的内容"""
    tags = [u'script', u'style', u'head', u'noscript']

    content = clear_comment(content)
    content = clear_tags_with_content(tags, content)
    content = clear_small_pic(content)
    return content


def http_join(url, content, tags=u'a|img', attrs=u'src|href', protocol=u'http|ftp'):
    """
    将内容中指定位置的url转为http开头的绝对路径
    根据items指定的参数，结合html结构抽取分别属性由双引号，单引号，后空格等形式的链接保存至字典
    遍历字典替换文本中的url为http url
    """
    pattern = u"(<({0}) [^>]*?({1})[\s=]*[\"']?([^>'\" ]*)[\"' ][^>]*?>)".format(tags, attrs)
    findall = regex.find_all(pattern, content)
    match_dict = dict()
    for match in findall:
        match_dict[match[0]] = match[3]
    _protocol = protocol.split(u'|')

    def __is_not_full(i, p):
        for _p in p:
            # if isinstance(i, unicode):
            # i = str(i)
            if i.startswith(_p):
                return True
        return False

    for k, v in match_dict.iteritems():
        if not __is_not_full(v, _protocol):
            http_k = urljoin(url, v)
            v2 = k.replace(v, http_k)
            content = content.replace(k, v2)
    return content


def rebuild_url(content, url):
    """
    重建URL，将URL转为绝对网址
    :param content:
    :param url:
    :return:
    """
    return http_join(url, content)


def clear_small_pic(content):
    """
    清除小图片
    :param content:
    :return:
    """
    pattern = u"(<img[^>]*?width\s*=\s*['\"](.+?)['\"][^>]*?>)"
    pics = re.findall(re.compile(pattern, re.I | re.M), content)
    if pics:
        for pic in pics:
            try:
                width = int(pic[1])
                if width < settings.MIN_IMAGE_WIDTH:
                    content = content.replace(pic[0], '')
            except:
                pass
    pattern = u"(<img[^>]*?height\s*=\s*['\"](.+?)['\"][^>]*?>)"
    pics = re.findall(re.compile(pattern, re.I | re.M), content)
    if pics:
        for pic in pics:
            try:
                width = int(pic[1])
                if width < settings.MIN_IMAGE_HEIGHT:
                    content = content.replace(pic[0], '')
            except:
                pass
    return content


def rebuild_img(content):
    """
    重建图片标签
    :param content:
    :return:
    """
    pattern = u"<img[^>]*?{0}\s*=\s*['\"](.+?)['\"][^>]*?>".format(settings.IMG_SRC)
    return regex.replace(pattern,
                         u'<img src="\g<2>" class="img-responsive">', content)


def images(content):
    """
    获取文章所有图片
    :param content:
    :return:
    """
    pattern = u'<img[^>]*?src=["](.+?)["].*?>'
    return list(set(regex.find_all(pattern, content)))


def rebuild(content, url):
    """
    重建URL和重建图片标签
    :param content:
    :param url:
    :return:
    """
    return rebuild_url(rebuild_img(content), url)


def format_content(content):
    """
    清除a，div换p
    :param content:
    :return:
    """
    content = clear_tag(u'a', content)
    content = regex.replace(u'<div.*?>', u'<p>', content)
    content = regex.replace(u'</div.*?>', u'</p>', content)
    content = regex.replace(u'<h1.*?>.*?</h1>', u'', content)
    content = regex.replace(u'>\s*<', u'><', content)
    content = regex.replace(u'<(b|strong|i)[^>]+?/>', u'', content)
    content = regex.replace(u'<img .*?>', u'\g<0><br/>', content)
    # content = regex.replace(u'(</?(p|br)>)+', u'\g<1>', content)
    # content = regex.replace(u'(</?(p|br)>*>)+', u'\g<1>', content)
    content = regex.replace(u'(</?br>)+', u'\g<1>', content)
    # content = regex.replace(u'(<p>\s*</p>)+', u'\g<1>', content)
    # content = regex.replace('[^"\']http://.+?\s*|</?strong>|</?b>|</?i>|</?em>', '', content)
    # content = regex.replace('[^"\']http://.+?\s*', '', content)
    content = regex.replace(u'声明[^><"]+', u'', content)
    content = regex.replace(u'\(?（?原标?题[^><"]+?<', u'<', content)
    content = regex.replace(u'>\s*[\w\d/\-_\.,\?&":\s]+?<', u'><', content)
    content = regex.replace(u'<(p|div)>(&nbsp;)*?</(p|div)>', '', content)
    # content = regex.replace('style=".*?"', '', content)


    # raise Exception(content)

    return content


def clear_content(content):
    content = regex.replace(u'<(?!img|a)(\w*?)\s.*?>', u'<\g<1>>', content)
    content = regex.replace(u'\s*?(<.*?>)\s*', u'\g<1>', content)
    content = regex.replace(u'<(/?)div.*?>', u'<\g<1>p>', content)
    content = regex.replace(u'<(/?)div.*?>', u'<\g<1>p>', content)
    content = regex.replace(u'(<p>)+', u'\g<1>', content)
    content = regex.replace(u'(</p>)+', u'\g<1>', content)
    content = regex.replace(u'<(/?)h\d+.*?>+', u'<\g<1>strong>', content)
    content = regex.replace(u'<p>\s*</p>|<p>.</p>|</?span>|<strong></strong>', u'', content)
    return content



# -*- coding: UTF-8 -*-


from gelid.extractors import html
from gelid.extractors.decorator import store


class Page(object):
    """页面HTML片断存储"""

    def __init__(self, source, url=None):
        self.url = url
        self.stores = {'html': source}

    @property
    @store
    def html(self):
        return None

    @property
    @store
    def header(self):
        return self._default_html(html.inner_text('header', self.html))

    def _default_html(self, value):
        if not value:
            return self.html
        else:
            return value

    @property
    @store
    def body(self):

        return self._default_html(html.inner_text('body', self.html))

    @property
    @store
    def title(self):
        return html.inner_text('title', self.html)

    @property
    @store
    def html_clean(self):
        return html.set_html_clean(self.html)

    @property
    @store
    def txt(self):
        return html.txt(self.html_clean)



# -*- coding: UTF-8 -*-
# lvjiyong on 2015/6/7.

# 当最大字数少于下述设置时，重新搜索内容
MAX_WORD_RESEARCH = 100

# 如果重新搜索的内容含有下述内容，则重新使用第一次找到内容
NOT_EXIST = [u'免责声明', u'本站注明', u'内容侵权', u'我们联系', u'联系我们', u'版权所有']

IMG_SRC = '(src|srcUrl|data-src|image-src|data-url|image-url|data|image)'

MIN_IMAGE_WIDTH = 200
MIN_IMAGE_HEIGHT = 160
REQUEST_MAX_PAGES = 30




# -*- coding: UTF-8 -*-
# lvjiyong on 2015/4/19.

# 部分代码参考与复制自 python-goose: https://github.com/grangier/python-goose
from collections import Counter
import random
import re
from gelidhttp.log import logger
from lxml import etree
import six

from gelid.extractors import html
from gelid.extractors.enums import ItemProp, Pack


class HtmlTree(object):
    def __init__(self, source, url=None):
        self.url = url
        self.stores = {Pack.html: etree.HTML(source, base_url=url)}

    @property
    def html(self):
        return self.stores.get(Pack.html)

    @property
    def author(self):
        """
        从itemprop获取作者名
        :return:
        """
        nodes = HtmlTree.item_prop(self.html, ItemProp.author.name)
        for node in nodes:
            name_nodes = HtmlTree.item_prop(node, ItemProp.name.name)
            if len(name_nodes) > 0:
                names = HtmlTree.texts(name_nodes[0])
                return u''.join([name.strip() for name in names if isinstance(name, six.string_types)])

    @staticmethod
    def item_prop(node, name):
        """
        从itemprop结构获取节点
        :param node:
        :param name:
        :return:
        """
        return HtmlTree.elements(node, attr=ItemProp.itemprop.name, value=name)

    @staticmethod
    def elements(node, tag=None, attr=None, value=None, child=False):
        """
        使用tag的attr属性值value获取节点
        :param node:原始节点数据
        :param tag:
        :param attr:
        :param value:
        :param child:
        :return:
        """
        ns = "http://exslt.org/regular-expressions"
        selector = 'descendant-or-self::%s' % (tag or '*')
        if attr and value:
            selector = '%s[re:test(@%s, "%s", "i")]' % (selector, attr, value)
        elems = node.xpath(selector, namespaces={"re": ns})
        if node in elems and (tag or child):
            elems.remove(node)
        return elems

    @staticmethod
    def texts(node):
        """
        获取节点内容
        :param node:
        :return:
        """
        return [i for i in node.itertext()]


def content_e(source, template):
    """
    获取两个html的非匹配元素
    :param source:
    :param template:
    :return:
    """
    tree1 = etree.HTML(clear_tag(source))
    tree2 = etree.HTML(clear_tag(template))

    e1, e2 = check_element(tree1, tree2)
    return e1


def content_html(e1):
    """
    获取元素html代码
    :param e1:
    :return:
    """

    return etree.tounicode(e1, method="html")


def clear_tag(source):
    """
    清除匹配时不需要的标签
    :param source:
    :return:
    """
    source = html.set_html_clean(source)
    source = re.sub('<(br|input).*?>', '', source)
    source = re.sub('<(ul|li|option|a|span|select|iframe|noscript|script|style)[^>]*?>[\w\W]*?</\\1>', '', source)
    source = re.sub('<([^>]+?) [^>]+?>\s*</\\1>', '', source)
    # p
    return source


def check_element(e1, e2):
    """
    比较两个lxml元素，直到分析至不同结构时返回
    :param e1:
    :param e2:
    :return:
    """
    l1 = len(e1)
    l2 = len(e2)
    if l1 and l1 != l2:
        return e1, e2  # e1.getparent()
    for i in range(0, l1):

        result = check_element(e1[i], e2[i])
        if result is not None:
            return result


def compare_pairs(item_count=10, max_count=20):
    """
    生成比较配对
    :param item_count:
    :param max_count:
    :return:
    """
    pairs = []
    rand_max_index = item_count - 1
    for i in range(1000):
        pair = (random.randint(0, rand_max_index), random.randint(0, rand_max_index))
        if pair not in pairs and (pair[1], pair[0]) not in pairs:
            pairs.append(pair)
            if len(pairs) >= max_count:
                break
    return pairs


def max_pattern(sources):
    """
    根据内容训练，查找最大数规则
    :param sources:
    :return:
    """
    sources_count = len(sources)
    pairs = compare_pairs(sources_count, 2*sources_count)
    patterns = []
    for pair in pairs:
        try:
            e = content_e(sources[pair[0]], sources[pair[1]])
            p1 = '//{0}[@id="{1}"]'
            p2 = '//{0}[@class="{1}"]'
            p3 = '//{0}[@id="{1}"][@class="{2}"]'
            p4 = '//{0}[@style="{1}"]'
            tag = e.tag
            id = e.attrib.get('id')
            cls = e.attrib.get('class')
            style = e.attrib.get('style')
            if id and cls:
                p = p3.format(tag, id, cls)
            elif id:
                p = p1.format(tag, id)
            elif cls:
                p = p2.format(tag, cls)
            elif style:
                p = p4.format(tag, style)
            else:
                p = ''
            patterns.append(p)
        except Exception,e:
            logger.warning(e)
            # logger.debug("S1-----------------------------------------")
            # logger.debug(sources[pair[0]])
            # logger.debug("S2-----------------------------------------")
            # logger.debug(sources[pair[1]])
            # logger.debug("SE-----------------------------------------")


    c = Counter(patterns)
    ps = c.most_common(1)
    return ps[0][0]



# -*- coding: utf-8 -*-
# Created by lvjiyong on 15/3/16
from os.path import dirname, join
from setuptools import setup, find_packages

with open(join(dirname(__file__), 'VERSION'), 'rb') as f:
    version = f.read().decode('ascii').strip()

setup(
    name="gelid",
    version=version,
    description=u"gelid专注中文网页内容抽取",
    author="lvjiyong",
    url="https://github.com/lvjiyong/python-gelid",
    license="Apache2.0",
    long_description=open('README.md').read(),
    maintainer='lvjiyong',
    platforms=["any"],
    maintainer_email='lvjiyong@gmail.com',
    include_package_data=True,
    packages=find_packages(exclude=('tests', 'docs')),
    install_requires=[
        'six',
        'lxml',
        'enum34',
        'readability-lxml',
        'jieba',
        'chardet',
        'gelid-http',
        'BeautifulSoup'
    ],
)


# -*- coding: UTF-8 -*-
# lvjiyong on 2015/4/26.

# -*- coding: UTF-8 -*-
# lvjiyong on 2015/4/3.
import math
import re
import time
import jieba
import six
from gelid.extractors import regex
from gelid.extractors import tm
from gelid.extractors import html
from gelidhttp.log import logger

class Score(object):
    """
    临时用于计分的，算法以新闻内容结合个人喜好进行评分
    """

    def __init__(self, article):
        content = article['content']
        self.article = article
        self.html = content  # .encode('utf-8')
        # 中文
        self.zh_text = None
        # 中文字数
        self.zh_count = 0
        # 图片数
        self.pic_count = 0
        # 段落数
        self.p_count = 0
        # 排名
        self.search_rank = 1
        # 网站指数
        self.site_rank = 1
        # 时间相关度，如果有时间date_rank= date - now = day，相关度-date_rank
        self.date_rank = 1
        # title相关度，以标题关键词出现次数为rank
        self.title_rank = 0

        self.default_rank = 0
        if content:
            self.default_rank = 0
            self.calculate()
        else:
            self.default_rank = -999999999

    def set_title_rank(self, title, keyword):
        """
        统计标题与关键词匹配分
        :param title:
        :param keyword:
        :return:
        """
        if not title and not self.html:
            self.title_rank = 0
        else:
            title = title.lower()
            keyword = keyword.lower()
            _rank = -2
            seg_title = jieba.cut(title)
            title_list = [t for t in seg_title]
            title_list = set(title_list)
            seg_keyword = jieba.cut(keyword)
            keyword_list = [t for t in seg_keyword]
            keyword_list = set(keyword_list)
            for k in keyword_list:
                for t in title_list:
                    if len(k) > 1 and k in t:
                        _rank += 2
            _rank *= 1.200
            if _rank == 0:
                _rank = -len(keyword_list)
            self.title_rank = _rank

    def calculate(self):
        """
        计算所有得分
        :return:
        """
        timestamp = self.article.get('posted_date')
        if timestamp:
            if isinstance(timestamp, six.integer_types):
                timestamp = float(timestamp)
            if isinstance(timestamp, float):
                self.set_date_rank(timestamp)
        self.set_zh_count()
        self.set_tag_rank()
        self.set_pic_count()
        title = self.article.get('title')
        keyword = self.article.get('keyword')
        if title and keyword:
            self.set_title_rank(title, keyword)

    def set_date_rank(self, timestamp):
        """
        计算时间分
        :param timestamp:
        :return:
        """
        if timestamp is None or timestamp == 0:
            self.date_rank = 0
        else:
            self.date_rank = (time.time() - timestamp) / (60.000 * 60 * 24)
            if self.date_rank<0:
                self.date_rank = 0


    def set_tag_rank(self):
        """
        通过P含量计处得分，暂未使用，将来将使用p量与每p文字量综合计算得分
        :return:
        """
        pattern = "<[^>]+>"
        result = re.findall(pattern, self.html, re.I | re.M)
        self.p_count = len(result) * 1.000

    def set_pic_count(self):
        """
        计算图片得分
        :return:
        """
        result = set([i for i in html.images(self.html) if not 'logo' in i])
        self.pic_count = len(result) * 1.000

    def set_zh_count(self):
        """
        计算中文字得分
        :return:
        """

        self.zh_text = regex.zh_cn(self.html)
        # logger.debug(self.zh_text)
        self.zh_count = len(self.zh_text) * 1.000

    @property
    def rank(self):
        """
        获取得分
        :return:
        """
        if self.default_rank < 0:
            return self.default_rank
        zh_score = math.sqrt(self.zh_count * 3)
        pic_score = math.sqrt(self.pic_count) * 20 if self.pic_count >0 else -100
        p_score =  - math.sqrt(self.p_count)
        search_score = math.sqrt(self.search_rank) * 10
        site_score = math.sqrt(self.site_rank) * 10
        # logger.debug(self.date_rank)
        date_score = - math.sqrt((math.log10(self.date_rank + 1) + 1)) * 5

        rank_score = zh_score + pic_score + p_score + site_score + search_score + date_score
        title_score = self.title_rank * math.sqrt(rank_score if rank_score >0 else 1)
        logger.debug(self.article.get('title'))

        logger.debug( pic_score)
        logger.debug( rank_score)
        logger.debug( title_score)
        logger.debug( '---------')

        return rank_score + title_score


def rank(articles):
    """
    批量计算文章，并以得分高低输出
    :param articles:
    :return:
    """
    ranks = [(article, Score(article).rank) for article in articles]
    ranks = sorted(ranks, key=lambda _rank: _rank[1], reverse=True)
    return ranks







# -*- coding: UTF-8 -*-
# lvjiyong on 2015/4/26.
import six


def title_in_content(title, content):
    """
    基于title文本扫描分析标题
    假设所有标题必会在内容中出现
    :return:
    """
    if title and content and isinstance(title, six.string_types):
        title = title.strip()
        for i in range(len(title)):
            exact_title = title[0:-i - 1]
            if exact_title and exact_title in content:
                title = exact_title
                break
    return title


# -*- coding: UTF-8 -*-
# lvjiyong on 2015/4/19.
from enum import Enum


class Pack(Enum):
    """内容存储片断Key"""
    title = 'title'
    html = 'html'
    text = 'text'
    header = 'header'
    body = "body"
    exact_title = 'exact_title'


class Node(Enum):
    """节点名"""
    title = 'title'
    head = 'head'
    body = 'body'


class ItemProp(Enum):
    """ItemProp名"""
    author = 'author'
    name = 'name'
    itemprop = 'itemprop'